!-----------------------------------------------------------------------------------------!
! Copyright (c) 2016 Peter Grünberg Institut, Forschungszentrum Jülich, Germany           !
! This file is part of kk-prime@juKKR and available as free software under the conditions !
! of the MIT license as expressed in the LICENSE file in more detail.                     !
!-----------------------------------------------------------------------------------------!


!-------------------------------------------------------------------------------
!> Summary: Module for timing measuremnts
!> Author: 
!> Deprecated: False ! This needs to be set to True for deprecated subroutines
!>
!-------------------------------------------------------------------------------
module mod_timing
  integer,parameter      :: nkeys=20
  integer,parameter      :: nkeylen=40
  character(len=nkeylen) :: timingkeys(nkeys)=''
  integer                :: start_time(nkeys)=0
  double precision       :: interm_time(nkeys)=0.0D0
  integer                :: ispaused(nkeys)=0
  integer                :: writetiming=1
  integer                :: init=0
contains 

!-------------------------------------------------------------------------------
!> Summary: Initialize timing files
!> Author: 
!> Category: PKKprime, profiling
!> Deprecated: False ! This needs to be set to True for deprecated subroutines
!>
!> Opens `timing.000.txt` and `timing.000.txt.test` files where `000` will be number of ranks that writes timing file
!-------------------------------------------------------------------------------
subroutine timing_init(my_rank)
  implicit none
  integer  :: my_rank
  character(len=3) :: ctemp
  if (init/=0) stop '[mod_timing] timing already initilized'
  write(ctemp,'(I03.3)') my_rank
  open(unit=43234059 , file='out_timing.'//trim(ctemp)//'.txt')
  open(unit=432340591, file='out_timing.'//trim(ctemp)//'.txt.test')
  init=1
end subroutine timing_init


!-------------------------------------------------------------------------------
!> Summary: Start timing measurment
!> Author: 
!> Category: PKKprime, profiling
!> Deprecated: False ! This needs to be set to True for deprecated subroutines
!>
!> 
!-------------------------------------------------------------------------------
subroutine timing_start(mykey2)
  implicit none
  integer time_array_1(8)
  integer ikey
  character(len=*)       :: mykey2
  character(len=nkeylen) :: mykey
!   integer,save           :: firstrun=1
! #ifdef MPI
!        INCLUDE "mpif.h"
! #endif

if (init/=1) stop '[timing_start] please run init first'
! write(*,*) mykey2

  mykey=mykey2

  ikey=timing_findkey(mykey,'noerror')

  if (ikey==-1) then
!   if  ( ispaused(ikey)==0 ) then
    ikey=timing_setkey(mykey)
    interm_time(ikey)=0.0D0
  else
    if  ( ispaused(ikey)==1) then
!     ikey=timing_findkey(mykey)
      if (ispaused(ikey)==0) stop '[timing_start] key already present unpaused'
    end if
  end if
!   write(*,*) '[timing_start]',mykey2,ispaused
!   write(*,*) '[timing_start]',ispaused
!   write(*,*) '[timing_start]',ispaused
!   write(*,*) '[timing_start]',timingkeys

   CALL SYSTEM_CLOCK(COUNT=start_time(ikey)) ! Start timing
!   call date_and_time(values=time_array_1)
!   start_time(ikey) = time_array_1 (5) * 3600 + time_array_1 (6) * 60 &
!                    + time_array_1 (7) + 0.001 * time_array_1 (8)
! #ifdef MPI
!   start_time(ikey) = MPI_Wtime()
! #endif
end subroutine timing_start

!-------------------------------------------------------------------------------
!> Summary: Pause timing measurement
!> Author: 
!> Category: PKKprime, profiling
!> Deprecated: False ! This needs to be set to True for deprecated subroutines
!>
!>
!-------------------------------------------------------------------------------
subroutine timing_pause(mykey2)
  implicit none
  integer time_array_1(8)
  integer    :: stop_time
  integer ikey
  character(len=*)       :: mykey2
  character(len=nkeylen) :: mykey
  real*8                   :: timing
!   real*8,optional          :: timeout
  integer                   :: clock_rate
! #ifdef MPI
!        INCLUDE "mpif.h"
! #endif

  mykey=mykey2
  ikey=timing_findkey(mykey)
!   write(*,*) '[timing_stop]',ikey


  CALL SYSTEM_CLOCK(COUNT_RATE=clock_rate) ! Find the rate
  CALL SYSTEM_CLOCK(COUNT=stop_time) ! Stop timing


!   call date_and_time(values=time_array_1)
!   stop_time = time_array_1 (5) * 3600 + time_array_1 (6) * 60 &
!             + time_array_1 (7) + 0.001 * time_array_1 (8)
! #ifdef MPI
!   stop_time = MPI_Wtime()
! #endif

  timing = (stop_time-start_time(ikey))/real(clock_rate)
  interm_time(ikey)=interm_time(ikey)+timing
  CALL SYSTEM_CLOCK(COUNT=start_time(ikey))
  ispaused(ikey)=1
! write(*,*) 'timingkeys',timingkeys
! write(*,*) 'start_time',start_time

end subroutine timing_pause

!-------------------------------------------------------------------------------
!> Summary: Stop timing measurement
!> Author: 
!> Category: PKKprime, profiling
!> Deprecated: False ! This needs to be set to True for deprecated subroutines
!>
!> @warning if cmode is given but not `test`, then no timing will be written. @endwarning
!-------------------------------------------------------------------------------
subroutine timing_stop(mykey2,cmode)
  implicit none
  integer time_array_1(8)
  integer    :: stop_time
  integer ikey
  character(len=*)       :: mykey2
  character(len=nkeylen) :: mykey
  real*8                   :: timing
!   real*8,optional          :: timeout
  integer                   :: clock_rate
  character(len=*),optional       :: cmode
  
! #ifdef MPI
!        INCLUDE "mpif.h"
! #endif
!   if ( .not. present(cmode) ) cmode='normal' 

  mykey=mykey2
  ikey=timing_findkey(mykey)
!   write(*,*) '[timing_stop]',ikey


  CALL SYSTEM_CLOCK(COUNT_RATE=clock_rate) ! Find the rate
  CALL SYSTEM_CLOCK(COUNT=stop_time) ! Stop timing


!   call date_and_time(values=time_array_1)
!   stop_time = time_array_1 (5) * 3600 + time_array_1 (6) * 60 &
!             + time_array_1 (7) + 0.001 * time_array_1 (8)
! #ifdef MPI
!   stop_time = MPI_Wtime()
! #endif

  timing = (stop_time-start_time(ikey))/real(clock_rate)+interm_time(ikey)

  call timing_delkey(mykey)

  if ( .not. present(cmode)) then
    write(43234059,*)  mykey,'  ',timing
  else if ( cmode=='test') then
    write(432340591,*) mykey,'  ',timing
!   else
!      print *,cmode
!      stop 'error in timing mode'
  end if

!   write(*,*) mykey,'  ',timing,interm_time(ikey)
!   if (present(timeout)) timeout=timing
  ispaused(ikey)=0

! write(*,*) 'timingkeys',timingkeys
! write(*,*) 'start_time',start_time

end subroutine timing_stop

!-------------------------------------------------------------------------------
!> Summary: Returns key index corresponding to key string
!> Author: 
!> Category: PKKprime, profiling
!> Deprecated: False ! This needs to be set to True for deprecated subroutines
!>
!> @warning returns -1 if optional argument `char2` is given and `char1` is not 
!> found in `timingkeys` @endwarning
!-------------------------------------------------------------------------------
integer function timing_findkey(char1,char2)
  implicit none
  character(len=nkeylen) :: char1
  character(len=*),optional :: char2
  integer                :: ival
  do ival = 1, nkeys
  if (char1==timingkeys(ival)) then
    timing_findkey=ival
    return
  end if
  end do
  if (.not. present(char2)) then 
    write(*,*) timingkeys
    write(*,*) '[timing_findkey] timing key ',trim(char1),' not found'
    stop 
  else
    timing_findkey=-1
  end if
end function timing_findkey

!-------------------------------------------------------------------------------
!> Summary: Sets new timingkey
!> Author: 
!> Category: PKKprime, profiling
!> Deprecated: False ! This needs to be set to True for deprecated subroutines
!>
!>
!-------------------------------------------------------------------------------
integer function timing_setkey(char1)
  implicit none
  character(len=nkeylen) :: char1
  integer                :: ival,ifound
  ifound=0
  do ival = 1, nkeys
    if (timingkeys(ival)==char1) then
      write(*,*) '[timing_setkey] key=',char1
      stop '[timing_setkey] error used twice the same timing key'

    end if
    if (trim(timingkeys(ival))=='' .and. ifound==0) then
      timingkeys(ival)=char1
      timing_setkey=ival
      ifound=1
    end if
  end do
  if (ifound==0) stop '[timing_setkey] not enough free spots to set timing values'
end function timing_setkey

!-------------------------------------------------------------------------------
!> Summary: Remove timingkey
!> Author: 
!> Category: PKKprime, profiling
!> Deprecated: False ! This needs to be set to True for deprecated subroutines
!>
!>
!-------------------------------------------------------------------------------
subroutine timing_delkey(char1)
  implicit none
  character(len=nkeylen) :: char1
  integer                :: ival

  do ival = 1, nkeys
  if (char1==timingkeys(ival)) then
    timingkeys(ival)=''
    start_time(ival)=0.0D0
    return
  end if
  end do
  stop '[timing_delkey] timing key not found'
end subroutine timing_delkey

end module mod_timing
